/*
 * libwpe-version.h
 * Copyright (C) 2021 Adrian Perez de Castro <aperez@igalia.com>
 *
 * Distributed under terms of the MIT license.
 */

#ifndef LIBWPE_VERSION_H
#define LIBWPE_VERSION_H

#define WPE_MAJOR_VERSION 1
#define WPE_MINOR_VERSION 10
#define WPE_MICRO_VERSION 0

#endif /* !LIBWPE_VERSION_H */
